<?php

require_once( 'class-plugin.php' );

/**
 * This class controls the WP UI, etc.
 * 
 * @package Mediaburst Ecommerce SMS
 * @since Alpha 1
 **/
class MBESMS extends MBESMS_Plugin {

	/**
	 * The version of this file, for the purposes of rewrite
	 * refreshes, JS/CSS cache bursting, etc.
	 *
	 * @var int
	 **/
	protected $version;

	// HOOKS
	// =====
	
	/**
	 * Initiate!
	 *
	 * @return void
	 **/
	public function __construct() {
		$this->setup( 'mbesms', 'plugin' );
		if ( is_admin() ) {
			$this->add_action( 'admin_init' );
			$this->add_action( 'admin_menu' );
		}
		$this->add_action( 'admin_bar_menu', null, 100 );
		$this->add_action( 'init', 'init_early', 0 );
		$this->add_action( 'parse_query' );
		$this->add_action( 'query_vars' );
		$this->add_action( 'mbesms_update_credits', 'update_credits' );
		$this->version = 1;
	}

	// HOOKS
	// =====

	/**
	 * Hooks the WP init action, nice and early.
	 *
	 * @return void
	 **/
	public function init_early() {
		$this->add_rewrite_rules();
		wp_enqueue_style( 'mbesms-general', $this->url( '/css/general.css' ), array(), $this->version, 'all' );
	}

	/**
	 * Hooks the admin_init action to 
	 *
	 * @return void
	 **/
	public function admin_init() {
		register_setting( 'mbesms', 'mbesms' );
		$this->maybe_update();
	}

	/**
	 * Hooks the WP admin_menu action to add in
	 * the options page for the Mediaburst API.
	 *
	 * @return void
	 **/
	public function admin_menu() {
		add_options_page( 'Mediaburst SMS API Settings', 'Mediaburst Account', 'manage_options', 'mbesms', array( & $this, 'options_page' ) );
	}

	/**
	 * Hooks the WP admin_bar_menu action to add a note about 
	 * current credit level. We cache the response from Mediaburst
	 * for 5 minutes.
	 *
	 * @param object $wp_admin_bar The WP Admin Bar, passed by reference
	 * @return void
	 **/
	public function admin_bar_menu( $wp_admin_bar ) {
		if ( ! current_user_can( 'manage_options' ) )
			return;
		if ( ! $this->get_option( 'credits_in_admin_bar' ) )
			return;
		// On the first run, we won't have this info until it's been loaded
		if ( ! ( $credits_info = $this->get_credits() ) )
			return;
		extract( $credits_info ); // Yields $timestamp and $credits
		$meta_title = sprintf( __( 'Credit balance updated on %1$s at %2$s. Click to purchase more credits on the Mediaburst API portal', 'mbesms' ), date( get_option( 'date_format' ), $timestamp ), date( get_option( 'time_format' ), $timestamp ) );
		$classes = "mbesms-credits-$credits";
		$args = array(
			'id' => 'mbesms_credits',
			'title' => sprintf( __( 'SMS Credits: %d', 'mbesms' ), $credits ),
			'href' => 'https://smsapi.mediaburst.co.uk/',
			'parent' => false,
			'meta' => array( 'title' => $meta_title, 'target' => '_blank', 'class' => $classes ),
		);
		$wp_admin_bar->add_menu( $args );
	}
	
	/**
	 * Hook the WP parse_query action, check if we're supposed to be 
	 * processing an upload or not.
	 *
	 * @param object $query A WP_Query object 
	 * @return void
	 **/
	public function parse_query( $query ) {
		// Check the request relates to us
		if ( ! (bool) get_query_var( 'mb_dlr_receipt' ) )
			return;
		$this->process_delivery_receipt();
		exit;
	}

	/**
	 * Hooks the WP query_vars filter to add in our 'mb_dlr_receipt' query var, which
	 * will flag that we want to process a delivery receipt.
	 *
	 * @param array $public_query_vars An array of GET vars which should be 
	 * acceptable to WP_Query.
	 * @return array An (amended) array of queryvars
	 **/
	public function query_vars( $public_query_vars ) {
		$public_query_vars[] = 'mb_dlr_receipt';
		return $public_query_vars;
	}
	
	/**
	 * Update the SMS balance, designed to hook a 
	 * WP Cron action, also to provide a value 
	 * on demand.
	 *
	 * @return void
	 **/
	public function update_credits() {
		$sms = $this->get_sms();
		$credits = (int) $sms->CheckCredit();
		$timestamp = current_time( 'timestamp' );
		$credits_info = array( 'credits' => $credits, 'timestamp' => $timestamp );
		update_option( 'mbesms_credits_info', $credits_info );
		return $credits_info;
	}

	// CALLBACKS
	// =========
	
	/**
	 * Callback function to provide the HTML for
	 * the options page.
	 *
	 * @return void
	 **/
	public function options_page() {
		$vars = array();
		$vars[ 'options' ] = $this->get_all_options();
		$this->render_admin( 'options.php', $vars );
	}
	
	// METHODS
	// =======

	/**
	 * Instantiate an instance of the Mediaburst SMS class
	 * on the sms property of this object.
	 *
	 * @return object A Mediaburst SMS object, all ready to go
	 **/
	public function get_sms() {
		$username = $this->get_option( 'username' );
		$password = $this->get_option( 'password' );
		try {
		    // Create a mediaburstSMS object
			$options = array(
				'http_class' => 'WordPressMBHTTP',
				'log' => true,
			);
		    $sms = new mediaburstSMS( $username, $password, $options );
		} catch (mediaburstException $e) {
		    error_log( "Exception creating SMS object (user: $username, pass: $password): ".$e->getMessage() );
			$sms = false;
		}
		return $sms;
	}
	
	/**
	 * Return the number of credits in a user's account,
	 * cached for 2 minutes.
	 *
	 * Returns an array like:
	 * Array (
	 * 		'timestamp' => 1318511673,
	 * 		'credits' => 153,
	 * )
	 *
	 * @return array The credit balance and timestamp
	 **/
	protected function get_credits() {
		$credits_info = get_option( 'mbesms_credits_info', false );
		extract( $credits_info ); // Yields $timestamp and $credits
		// Is it older than 2 mins?
		$now = current_time( 'timestamp' );
		// If the credit balance is more than 10 minutes old, update immediately
		if ( ! $credits_info || $now > ( (int) @ $timestamp + ( 60 * 10 ) ) ) {
			$credits_info = $this->update_credits();
			extract( $credits_info, EXTR_OVERWRITE ); // Yields $timestamp and $credits, overwriting existing
		// If the credit balance is more than 2 minutes old, schedule a background update
		} elseif ( $now > ( (int) @ $timestamp + ( 60 * 2 ) ) ) {
			wp_schedule_single_event( time(), 'mbesms_update_credits' );
		}
		if ( ! $credits_info )
			return false;
		return $credits_info;
	}
	
	/**
	 * Adds the rewrite rules.
	 *
	 * @return void
	 **/
	protected function add_rewrite_rules() {
		$regex = 'mb-dlr-receipt';
		$redirect = 'index.php?mb_dlr_receipt=1';
		add_rewrite_rule( $regex, $redirect, 'top' );
	}

	/**
	 * Process a delivery receipt which has arrived by POST.
	 *
	 * @return void
	 **/
	protected function process_delivery_receipt() {
		$plugin = @ $_POST[ 'PLUGIN' ];
		// We cannot proceed without a plugin to handle 
		// this request
		if ( ! $plugin )
			return;
		// N.B. Note double $ below
		global $$plugin;
		$$plugin->handle_delivery_receipt();
		exit;
	}
	
	/**
	 * Checks the DB structure is up to date, flushes rewrite rules, etc.
	 *
	 * @return void
	 **/
	protected function maybe_update() {
		global $wpdb;
		$option_name = 'mbesms-version';
		$version = get_option( $option_name, 0 );

		if ( $this->version == $version )
			return;

		if ( $version < 1 ) {
			flush_rewrite_rules();
			error_log( "MBES: Flush rewrite rules" );
		}

		error_log( "MBES: Done updates" );
		update_option( $option_name, $this->version );
	}

} // END MBE2S class 

$mbesms = new MBESMS();

?>