<?php

require_once( 'class-plugin.php' );

/**
 * A parent class from which classes targetting specific 
 * ecommerce plugins can inherit.
 *
 * @package Mediaburst Ecommerce SMS
 * @since Alpha 1
 */
class MBESMS_Generic extends MBESMS_Plugin {

	/**
	 * The version of this file, for the purposes of rewrite
	 * refreshes, JS/CSS cache bursting, etc.
	 *
	 * @var int
	 **/
	protected $version;

	/**
	 * The Mediaburst SMS API class
	 *
	 * @var object
	 **/
	protected $sms;

	// HOOKS
	// =====
	
	/**
	 * Initiate!
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function __construct() {
		if ( is_admin() )
 			$this->add_action( 'admin_init', '_admin_init' );
		// $this->test_format_mobile_number();
	}

	/**
	 * Hooks the admin_init action to run the update method.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function _admin_init() {
		$this->maybe_update();
	}
	
	// METHODS
	// =======

	/**
	 * This function must be implemented by the child class.
	 *
	 * @return void
	 **/
	public function handle_delivery_receipt() {
		throw new exception( 'Please override the "handle_delivery_receipt" function in the child class.' );
	}

	/**
	 * Converts a long string Mediaburst date to a Unix timestamp,
	 * assuming that the date is UTC. Example of date:
	 * 20111013151916 (equates to 2011/10/13 15:19:16)
	 *
	 * @param string $mediaburst_date A date in the Mediaburst API's long run-together format 
	 * @param bool $gmt Whether to return the date in local time or GMT/UTC
	 * @return int A UNIX timestamp
	 **/
	protected function mediaburst_date_to_timestamp( $mediaburst_date, $gmt = false ) {
		$date_regex = '/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})$/';
		preg_match( $date_regex, $mediaburst_date, $matches );
		foreach ( $matches as & $match )
			$match = ltrim( $match, '0' );
		list( $date, $yr, $mth, $d, $hr, $min, $sec ) = $matches;
		$timestamp = mktime( $hr, $min, $sec, $mth, $d, $yr );
		if ( ! $gmt )
			$timestamp += get_option( 'gmt_offset' ) * 3600;
		return $timestamp;
	}

	/**
	 * Removes any characters which are not alphanumeric,
	 * including spaces.
	 *
	 * @param string $from The SMS From string to sanitise 
	 * @return string A sanitised SMS From string
	 **/
	protected function sanitise_from_str( $from ) {
		return preg_replace( '/[^a-z0-9]/i', '', $from );
	}

	/**
	 * Takes a mobile number and a country and makes it all nice
	 * and ready for the API:
	 * * Remove leading '0'
	 * * Detect or add country code
	 * * Strip any weird characters and spaces
	 *
	 * @param string $mobile_number The mobile number to fixup
	 * @param string $country_isocode The two letter ISO country code to get the dialling prefix for
	 * @return string A formatted mobile phone number
	 **/
	protected function format_mobile_number( $mobile_number, $country_isocode ) {
		$seq = $mobile_number;

		// First remove any whitespace
		$start = $mobile_number;
		$mobile_number = preg_replace( '/\s/', '', $mobile_number );

		$country_dial = $this->get_country_calling_code( $country_isocode );

		// Attempt to detect a country prefix by looking for:
		// * "+" at the start of the mobile number
		// * Any digits preceding a parentheses, e.g. "44(0)797…"
		if ( '+' != substr( $mobile_number, 0, 1 ) && ! preg_match( '/^\d+\(/', $mobile_number ) ) {

			// Strip any leading zero
			$mobile_number = preg_replace( '/^0/', '', $mobile_number );	

			// No country code detected, so add one

			if ( $country_dial != substr( $mobile_number, 0, strlen( $country_dial ) ) ) {
				// The number doesn't start with the country code, so add it now
				$mobile_number = $country_dial . $mobile_number;
			}

		}

		// Remove parentheses and anything betwixt them
		$mobile_number = preg_replace( '/\(\d+\)/', '', $mobile_number );

		// Remove anything that isn't a number
		$mobile_number = preg_replace( '/[^0-9]/', '', $mobile_number );

		// The number starts with the expected country code, remove any zero which 
		// immediately follows the country code.
		if ( $country_dial == substr( $mobile_number, 0, strlen( $country_dial ) ) )
			$mobile_number = preg_replace( "/^{$country_dial}(\s*)?0/", $country_dial, $mobile_number );

		return $mobile_number;
	}

	/**
	 * Test the current format_mobile_number method against an
	 * array of user input numbers and the known good answers.
	 *
	 * @return void
	 **/
	protected function test_format_mobile_number() {
		$nums = array(
			array( '07971687295', 'uk', '447971687295' ),
			array( '+4407971687295', 'uk', '447971687295' ),
			array( '447971687295', 'uk', '447971687295' ),
			array( '+44(0)7971687295', 'uk', '447971687295' ),
			array( '07971 687295', 'uk', '447971687295' ),
			array( '44 0 7971 687295', 'uk', '447971687295' ),
		);
		echo "<table><thead><tr><th scope='col'>Input</th><th scope='col'>Country</th><th scope='col'>Output</th><th scope='col'>Correct?</th></tr></thead><tbody>";
		foreach ( $nums as $n ) {
			$num = $n[ 0 ];
			$code = $n[ 1 ];
			$fnum = $this->format_mobile_number( $num, $code );
			$correct = ( $fnum === $n[ 2 ] ) ? '<span style="color: green;">correct</span>' : '<span style="color: red;">incorrect</span>' ;
			echo "<tr><td><kbd>$num</kbd></td><td><kbd>$code</kbd></td><td><kbd>$fnum</kbd></td><td><kbd>$correct</kbd></td></tr>";
		}
		echo "</tbody></table>";
		exit;
	}

	/**
	 * Returns the dialling prefix for a country when passed 
	 * the ISO country code.
	 *
	 * @param string $iso_code The two character ISO country code, e.g. UK 
	 * @return string The dialling prefix
	 **/
	protected function get_country_calling_code( $iso_code ) {
		require( 'country-calling-codes.php' );
		$iso_code = strtoupper( $iso_code );
		if ( ! isset( $country_codes[ $iso_code ] ) )
			return '';
		return $country_codes[ $iso_code ];
	}

	/**
	 * Wraps the Mediaburst SMS class to send an SMS message.
	 *
	 * @param string|array $numbers Either a single number, with international code, or an array of them
	 * @param string $message The message to send by SMS 
	 * @param bool $delivery_receipt Whether we wish to receive a delivery receipt or not
	 * @return void
	 **/
	protected function send_sms( $numbers, $message, $delivery_receipt = false ) {
		global $mbesms;
		$numbers = (array) $numbers;
		
		// Params docs: http://www.mediaburst.co.uk/api/sending-a-message/parameters/
		
		$add_params = array(
			'ClientID' => 'wp_' . uniqid(),
		);
		
		if ( $from = $this->get_option( 'from' ) )
			$add_params[ 'From' ] = $from;
		
		if ( $delivery_receipt ) {
			// Delivery receipt params docs: http://www.mediaburst.co.uk/api/sending-a-message/delivery-receipts/
			$delivery_receipt = array_merge(
				array( // Defaults
					'CLIENT_ID' => '#CLIENT_ID#',
					'STATUS' => '#DELIVERY_STATUS#',
					'DEST_ADDR' => '#DEST_ADDR#',
					'ERR_CODE' => '#ERR_CODE#',
					'MSG_ID' => '#MSG_ID#',
					'SRC_ADDR' => '#SRC_ADDR#',
					'SUBMIT_DATE' => '#SUBMIT_DATE#',
					'STATUS_DATE' => '#STATUS_DATE#',
				),
				$delivery_receipt
			);
			$add_params[ 'DlrType' ] = 2; // POST request
			$add_params[ 'DlrUrl' ] = home_url( 'mb-dlr-receipt' );
			$dlr_content = http_build_query( $delivery_receipt, null, '&amp;' );
			// This encodes the #s, which Mediaburst liketh not
			$dlr_content = str_replace( '%23', '#', $dlr_content );
			$add_params[ 'DlrContent' ] = $dlr_content;
		}
	
		if ( ! $sms = $mbesms->get_sms() )
			return false;
	
		try {
		    // Send a text message
		    $result = $sms->Send( $numbers, $message, $add_params );
		} catch (mediaburstException $e) {
			error_log( 'Exception sending SMS: '.$e->getMessage() );
			$result = array();
			$result[ 0 ] = array(
				'success' => false,
				'id' => '',
				'error_message' => $e->getMessage(),
				'to' => $numbers[ 0 ],
				'message' => $message,
				'clientid' => $add_params[ 'ClientID' ],
			);
			return $result;
		}
		return $result;
	}

	/**
	 * Should be overriden by the child class.
	 *
	 * @return void
	 **/
	protected function maybe_update() {
		throw new exception( 'Please override the "maybe_update" function in the child class.' );
	}

} // END MBESMS_Generic class 

?>